@@ -1,3 +1,24 @@
+#
+#   xts: eXtensible time-series 
+#
+#   Copyright (C) 2008  Jeffrey A. Ryan jeff.a.ryan @ gmail.com
+#
+#   Contributions from Joshua M. Ulrich
+#
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+
 # to.period functionality from quantmod
 #
 # to.period base function
@@ -12,7 +33,7 @@
 # endpoints needs to be able to handle 'k' arguments to minutes...
 
 `to.period` <-
-function(x,period='months',k=1,indexAt=NULL,name=NULL,...)
+function(x,period='months',k=1,indexAt=NULL,name=NULL,OHLC=TRUE,...)
 {
   original.class <- class(x) # used for classes derived from 'xts'
 
